function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/*
 * Choice JS
*/
class Builder {
  //Constructor
  constructor(target, userOptions) {
    this.target = target; //Build our default options

    this.options = _objectSpread(_objectSpread({}, this.buildDefaultOptions(userOptions)), userOptions);
  }
  /**
   * Create a new choice dropdown
   * 
   * @return {Promise}
  */


  create() {
    var _this = this;

    return _asyncToGenerator(function* () {
      //Build it
      var choice = yield _this.build(); //Toast bindings

      yield _this.bindings(choice); //Deploy the select

      _this.target.parentNode.insertBefore(choice, _this.target.nextSibling); //Dispatch event to load it


      _this.target.dispatchEvent(new Event('change')); //Done


      return choice;
    })();
  }
  /**
   * Build the default options
   * 
   * @param {Object} userOptions The user options for any overwrites
   * @return {Object} An object of options
  */


  buildDefaultOptions(userOptions) {
    var options = {
      width: 'inherit',
      height: 'inherit',
      placeholder: this.target.hasAttribute('placeholder') ? this.target.getAttribute('placeholder') : false,
      class: '',
      closeOnSelect: false,
      event: 'change',
      copyAttributes: true
    }; //Return

    return options;
  }
  /**
   * Configure options for the choice
   * 
   * @param {HTMLElement} choice - The select we're configuring options for
   * @return {Promise}
  */


  configureOptions(choice) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      //Width and height
      choice.style.width = _this2.options.width == 'inherit' ? _this2.target.offsetWidth + 'px' : _this2.options.width, choice.style.minHeight = _this2.options.height == 'inherit' ? _this2.target.offsetHeight + 'px' : _this2.options.height; //Class

      if (_this2.options.class != '') choice.classList.add(_this2.options.class); //Placeholder

      if (_this2.options.placeholder) choice.querySelector('.choicejs-select').innerHTML = _this2.options.placeholder; //Return

      return;
    })();
  }
  /**
   * Add option
   *
   * @param {HTMLElement} option - The option we're selecting
   * @returns {Promise}
  */


  addOption(option) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      //Get elems
      var value = option.getAttribute('value'),
          multiple = _this3.target.hasAttribute('multiple') && ['true', 'multiple'].includes(_this3.target.getAttribute('multiple')); //Mark the option as selected

      _this3.target.querySelector("option[value=\"".concat(value, "\"]")).selected = true; //If its not multiple, set the value of the parent select box

      if (!multiple) {
        _this3.target.value = value;
      }
    })();
  }
  /**
   * Remove option
   *
   * @param {HTMLElement} option - The option we're selecting
   * @returns {Promise}
  */


  removeOption(option) {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      //Get elems
      var value = option.getAttribute('value'),
          multiple = _this4.target.hasAttribute('multiple') && ['true', 'multiple'].includes(_this4.target.getAttribute('multiple')); //Remove the hidden option

      _this4.target.querySelector("option[value=\"".concat(value, "\"]")).selected = false;

      _this4.target.querySelector("option[value=\"".concat(value, "\"]")).removeAttribute('selected'); //Single or multiple select?


      if (multiple) {
        //If there is still options selected, return now, don't reset the value and index of the select
        if (_this4.target.querySelectorAll("option:checked").length > 0) return;
      } //Reset


      _this4.target.value = '';
      _this4.target.selectedIndex = -1;
    })();
  }
  /* ------------------------------------------------------------------------------------------------------------------------------------------------
  |  HTML Builders
  ------------------------------------------------------------------------------------------------------------------------------------------------ */

  /**
   * Build the choice
   * 
   * @return {Promise} HTMLElement The new choice
  */


  build() {
    var _this5 = this;

    return _asyncToGenerator(function* () {
      //Build the new Select HTML element and add it directly after the target
      var wrapper = document.createElement('div'),
          select = document.createElement('div'),
          search = document.createElement('input'),
          dropdown = document.createElement('div'),
          list = document.createElement('ul'); //Set attributes

      wrapper.classList.add('choicejs');
      wrapper.setAttribute('id', 'choice-' + Math.random().toString(36).substr(2, 4));
      select.classList.add('choicejs-select');
      search.classList.add('choicejs-search');
      search.setAttribute('placeholder', 'Search');
      dropdown.classList.add('choicejs-dropdown'); //Set the id on the target so we can relate

      _this5.target.setAttribute('data-choicejs-id', "#".concat(wrapper.id)); //Copy the select boxes attributes


      if (_this5.options.copyAttributes === true) {
        //Only copy over limited attributes
        wrapper.setAttribute('name', _this5.target.getAttribute('name'));
      } //Loop the targets options and build our list


      for (var option of _this5.target.querySelectorAll('option')) {
        if (option.value != '' && option.value != undefined) {
          var item = document.createElement('li');
          item.innerHTML = option.innerHTML; //Copy all attributes over from the option

          if (_this5.options.copyAttributes === true) {
            for (var i = 0; i < option.attributes.length; i++) {
              item.setAttribute(option.attributes[i].nodeName, option.attributes[i].nodeValue);
            }
          }

          list.appendChild(item);
        }
      } //Build the whole select and return it


      dropdown.appendChild(search);
      dropdown.appendChild(list);
      wrapper.appendChild(select);
      wrapper.appendChild(dropdown); //Configure the options for it

      yield _this5.configureOptions(wrapper); //Return

      return wrapper;
    })();
  }
  /**
   * Build option pill
   *
   * @param {Node} option - The option selected
   * @param {Node} select - The select we're appending to
   * @returns {Promise}
  */


  addOptionPill(option, select) {
    var _this6 = this;

    return _asyncToGenerator(function* () {
      var wrapper = document.createElement('div'),
          close = document.createElement('span'),
          value = option.getAttribute('value'); //Attributes

      wrapper.classList.add('choicejs-pill');
      wrapper.setAttribute('value', value);
      wrapper.innerHTML = option.innerHTML; //Build it

      wrapper.appendChild(close);
      select.appendChild(wrapper); //Add the remove event listener

      close.addEventListener('click', /*#__PURE__*/function () {
        var _ref = _asyncToGenerator(function* (e) {
          //Run remove btn
          yield _this6.removeOption(option); //What parent event to trigger?

          if (_this6.options.event !== false) {
            _this6.target.dispatchEvent(new Event(_this6.options.event));
          }
        });

        return function (_x) {
          return _ref.apply(this, arguments);
        };
      }()); //Return

      return;
    })();
  }
  /* ------------------------------------------------------------------------------------------------------------------------------------------------
  |  Bindings
  ------------------------------------------------------------------------------------------------------------------------------------------------ */

  /**
   * Bind Event Listener
   * 
   * @param {HTMLElement} choice - The choice we're binding
   * @return {Promise}
  */


  bindEventListener(choice) {
    var _this7 = this;

    return _asyncToGenerator(function* () {
      //When an event is fired on the target, change our dropdowns
      _this7.target.addEventListener(_this7.options.event, /*#__PURE__*/_asyncToGenerator(function* () {
        //Rebuild the options and pills on the target change
        var multiple = _this7.target.hasAttribute('multiple') && ['true', 'multiple'].includes(_this7.target.getAttribute('multiple')),
            choice_select = choice.querySelector('.choicejs-select'); //Reset the select placeholder

        choice_select.innerHTML = _this7.options.placeholder ? _this7.options.placeholder : ''; //Reset all of the options

        for (var option of choice.querySelectorAll(".choicejs-dropdown ul li")) {
          option.removeAttribute('selected');
        } //Are we working with multiple?


        if (!multiple) {
          var selected_value = _this7.target.value; //Does the select have a value?

          if (selected_value != '') {
            var choice_option = choice.querySelector(".choicejs-dropdown ul li[value=\"".concat(selected_value, "\"]")); //Select it

            choice_option.setAttribute('selected', true);
            choice_select.innerHTML = choice_option.innerHTML;
          }
        } else {
          var pills = [],
              selected_options = _this7.target.querySelectorAll('option:checked');

          if (selected_options.length > 0) {
            choice_select.innerHTML = '';

            for (var _option of selected_options) {
              var _choice_option = choice.querySelector(".choicejs-dropdown ul li[value=\"".concat(_option.value, "\"]"));

              _choice_option.setAttribute('selected', true); //Add a pillbox


              pills.push(_this7.addOptionPill(_choice_option, choice_select));
            }
          }

          yield Promise.all(pills);
        }
      }));
    })();
  }
  /**
   * Bind active click
   *
   * @param {HTMLElement} choice - The choice we're binding
   * @return {Promise}
  */


  bindActiveListener(choice) {
    return _asyncToGenerator(function* () {
      choice.addEventListener('click', () => {
        choice.classList.add('active');
      });
      document.addEventListener('click', function (event) {
        if (!choice.contains(event.target)) {
          choice.classList.remove('active');
        }
      }); //Return

      return;
    })();
  }
  /**
   * Bind option select
   * 
   * @param {HTMLElement} choice - The choice we're binding
   * @return {Promise}
  */


  bindOptionSelect(choice) {
    var _this8 = this;

    return _asyncToGenerator(function* () {
      var _loop = function _loop(option) {
        //Add the event listener
        option.addEventListener('click', /*#__PURE__*/function () {
          var _ref3 = _asyncToGenerator(function* (e) {
            //Stop Prop
            e.stopPropagation();
            e.stopImmediatePropagation(); //Add or remove

            if (option.hasAttribute('selected')) {
              //Remove the option
              yield _this8.removeOption(option);
            } else {
              //Add the option
              yield _this8.addOption(option);
            } //Close on select?


            if (_this8.options.closeOnSelect) {
              choice.classList.remove('active');
            } //What parent event to trigger?


            if (_this8.options.event !== false) {
              _this8.target.dispatchEvent(new Event(_this8.options.event));
            }
          });

          return function (_x2) {
            return _ref3.apply(this, arguments);
          };
        }());
      };

      //Loop and add
      for (var option of choice.querySelectorAll('ul li')) {
        _loop(option);
      } //Return


      return;
    })();
  }
  /**
   * Bind option filtering. When a user types into the search box
   * 
   * @param {HTMLElement} choice - The choice we're binding
   * @return {Promise}
  */


  bindOptionFiltering(choice) {
    return _asyncToGenerator(function* () {
      //When the choice search bar changes
      var searchTimeout = 0,
          searchBar = choice.querySelector('.choicejs-search'),
          options = choice.querySelectorAll('ul li'); //When the postcode is changed, test it

      searchBar.addEventListener('input', () => {
        //Clear the above timeout
        clearTimeout(searchTimeout); //Set a new timeout

        searchTimeout = setTimeout(() => {
          //Loop all options
          for (var option of options) {
            if (option.innerHTML.toLowerCase().includes(searchBar.value.toLowerCase())) {
              option.classList.remove('hide');
            } else {
              option.classList.add('hide');
            }
          }
        }, 500);
      }); //Return

      return;
    })();
  }
  /**
   * Add all of the bindings for the choice
   * 
   * @param {HTMLElement} choice - The select we're building the bindings for
   * @return {Promise}
  */


  bindings(choice) {
    var _this9 = this;

    return _asyncToGenerator(function* () {
      //Return
      return yield Promise.all([_this9.bindActiveListener(choice), _this9.bindOptionSelect(choice), _this9.bindOptionFiltering(choice), _this9.bindEventListener(choice)]);
    })();
  }

}
/**
 * Export the choice function to use
 *
 * @param {NodeElement} target - The target we're adding the choice to
 * @param {Object} options - The options
 * @returns {Promise}  - An instance of choice
 */


export var Choice = /*#__PURE__*/function () {
  var _ref4 = _asyncToGenerator(function* (target, options) {
    //Start a new Choice
    var newBuild = new Builder(target, options); //Run the build

    return yield newBuild.create();
  });

  return function Choice(_x3, _x4) {
    return _ref4.apply(this, arguments);
  };
}();